use crate::jira_config::JiraConfig;
use reqwest::header::{HeaderMap, HeaderValue, CONTENT_TYPE};
use serde::{Deserialize, Serialize};
use std::fs;
use std::io::Write;
use std::path::PathBuf;
use tempfile::NamedTempFile;

#[derive(Debug, Serialize)]
struct JiraIssueRequest {
    fields: JiraIssueFields,
}

#[derive(Debug, Serialize)]
struct JiraIssueFields {
    project: Project,
    summary: String,
    description: String,
    issuetype: IssueType,
}

#[derive(Debug, Serialize)]
struct Project {
    key: String,
}

#[derive(Debug, Serialize)]
struct IssueType {
    name: String,
}

#[derive(Deserialize)]
struct JiraResponse {
    key: String,
}

#[derive(Deserialize)]
struct JiraSearchResponse {
    issues: Vec<JiraIssue>,
    total: u32,
}

#[derive(Deserialize)]
struct JiraIssue {
    key: String,
    fields: JiraIssueResponseFields,
}

#[derive(Deserialize)]
struct JiraIssueResponseFields {
    summary: String,
    status: Status,
    created: String,
}

#[derive(Deserialize)]
struct Status {
    name: String,
}

fn get_editor() -> String {
    std::env::var("EDITOR")
        .or_else(|_| std::env::var("VISUAL"))
        .unwrap_or_else(|_| "vi".to_string())
}

fn parse_markdown(content: &str) -> (String, String) {
    let mut lines = content.lines();

    // Get first non-empty line as title
    let title = lines
        .by_ref()
        .find(|line| !line.trim().is_empty())
        .unwrap_or("")
        .trim_start_matches('#')
        .trim()
        .to_string();

    // Rest is description
    let description = lines.collect::<Vec<&str>>().join("\n");

    (title, description)
}

fn create_temp_markdown() -> Result<String, Box<dyn std::error::Error>> {
    let mut temp_file = NamedTempFile::new()?;

    let template = r#"# "#;

    temp_file.write_all(template.as_bytes())?;
    temp_file.flush()?;

    let editor = get_editor();
    let status = std::process::Command::new(editor)
        .arg(temp_file.path())
        .status()?;

    if !status.success() {
        return Err("Editor exited with non-zero status".into());
    }

    // Read the modified content
    let content = fs::read_to_string(temp_file.path())?;

    // temp_file will be automatically deleted when it goes out of scope
    Ok(content)
}

async fn create_jira_issue(
    _config: &JiraConfig,
    project_key: &str,
    title: &str,
    description: &str,
) -> Result<JiraResponse, Box<dyn std::error::Error>> {
    let _client = reqwest::Client::new();

    let mut headers = HeaderMap::new();
    headers.insert(CONTENT_TYPE, HeaderValue::from_static("application/json"));

    let issue = JiraIssueRequest {
        fields: JiraIssueFields {
            project: Project {
                key: project_key.to_string(),
            },
            summary: title.to_string(),
            description: description.to_string(),
            issuetype: IssueType {
                name: "Task".to_string(),
            },
        },
    };

    println!("{:#?}", issue);

    Ok(JiraResponse {
        key: "123".to_string(),
    })

    // let response = client
    //     .post(format!("{}/rest/api/2/issue", config.url))
    //     .basic_auth(&config.email, Some(&config.api_token))
    //     .headers(headers)
    //     .json(&issue)
    //     .send()
    //     .await?;
    //
    // if !response.status().is_success() {
    //     let error_text = response.text().await?;
    //     return Err(format!("Failed to create issue: {}", error_text).into());
    // }
    //
    // Ok(response.json::<JiraResponse>().await?)
}

pub async fn create(
    project: String,
    markdown_file: Option<PathBuf>,
) -> Result<(), Box<dyn std::error::Error>> {
    let config = JiraConfig::load().map_err(|e| format!("Configuration error: {}", e))?;

    let content = match markdown_file {
        Some(path) => {
            fs::read_to_string(&path).map_err(|e| format!("Failed to read file: {}", e))?
        }
        None => {
            println!("No markdown file specified. Opening editor...");
            create_temp_markdown()?
        }
    };

    if content.trim().is_empty() {
        return Err("Empty content. Aborting ticket creation.".into());
    }

    let (title, description) = parse_markdown(&content);

    // Confirm creation
    println!("\nAbout to create ticket:");
    println!("Title: {}", title);
    println!("{}...", &description.chars().take(100).collect::<String>());
    println!("\nPress Enter to continue or Ctrl+C to abort");

    let mut input = String::new();
    std::io::stdin().read_line(&mut input)?;

    let response = create_jira_issue(&config, &project, &title, &description).await?;

    println!("Successfully created ticket: {}", response.key);
    println!("URL: {}/browse/{}", config.url, response.key);

    Ok(())
}
